@@ -21,11 +21,12 @@
 #include <framework/mlt_filter.h>
 #include <framework/mlt_frame.h>
 #include <framework/mlt_log.h>
+#include <framework/mlt_profile.h>
 
 // ffmpeg Header files
-#include <avformat.h>
+#include <libavformat/avformat.h>
 #ifdef SWSCALE
-#include <swscale.h>
+#include <libswscale/swscale.h>
 #endif
 
 #if LIBAVUTIL_VERSION_INT < (50<<16)
@@ -72,10 +73,46 @@ static int convert_mlt_to_av_cs( mlt_image_format format )
 	return value;
 }
 
-static void av_convert_image( uint8_t *out, uint8_t *in, int out_fmt, int in_fmt, int width, int height )
+static void set_luma_transfer( struct SwsContext *context, int colorspace, int use_full_range )
+{
+#if defined(SWSCALE) && (LIBSWSCALE_VERSION_INT >= ((0<<16)+(7<<8)+2))
+	int *coefficients;
+	int full_range;
+	int brightness, contrast, saturation;
+
+	if ( sws_getColorspaceDetails( context, &coefficients, &full_range, &coefficients, &full_range,
+			&brightness, &contrast, &saturation ) != -1 )
+	{
+		// Don't change these from defaults unless explicitly told to.
+		if ( use_full_range >= 0 )
+			full_range = use_full_range;
+		switch ( colorspace )
+		{
+		case 170:
+		case 470:
+		case 601:
+		case 624:
+			coefficients = sws_getCoefficients( SWS_CS_ITU601 );
+			break;
+		case 240:
+			coefficients = sws_getCoefficients( SWS_CS_SMPTE240M );
+			break;
+		case 709:
+			coefficients = sws_getCoefficients( SWS_CS_ITU709 );
+			break;
+		}
+		sws_setColorspaceDetails( context, coefficients, full_range, coefficients, full_range,
+			brightness, contrast, saturation );
+	}
+#endif
+}
+
+static void av_convert_image( uint8_t *out, uint8_t *in, int out_fmt, int in_fmt,
+	int width, int height, int colorspace, int use_full_range )
 {
 	AVPicture input;
 	AVPicture output;
+#ifdef SWSCALE
 	int flags = SWS_BILINEAR | SWS_ACCURATE_RND;
 
 	if ( out_fmt == PIX_FMT_YUYV422 )
@@ -88,12 +125,14 @@ static void av_convert_image( uint8_t *out, uint8_t *in, int out_fmt, int in_fmt
 #ifdef USE_SSE
 	flags |= SWS_CPU_CAPS_MMX2;
 #endif
+#endif /* SWSCALE */
 
 	avpicture_fill( &input, in, in_fmt, width, height );
 	avpicture_fill( &output, out, out_fmt, width, height );
 #ifdef SWSCALE
 	struct SwsContext *context = sws_getContext( width, height, in_fmt,
 		width, height, out_fmt, flags, NULL, NULL, NULL);
+	set_luma_transfer( context, colorspace, use_full_range );
 	sws_scale( context, input.data, input.linesize, 0, height,
 		output.data, output.linesize);
 	sws_freeContext( context );
@@ -114,9 +153,12 @@ static int convert_image( mlt_frame frame, uint8_t **image, mlt_image_format *fo
 
 	if ( *format != output_format )
 	{
-		mlt_log_debug( NULL, "[filter avcolor_space] %s -> %s @ %dx%d\n",
+		int colorspace = mlt_properties_get_int( properties, "colorspace" );
+		int force_full_luma = -1;
+		
+		mlt_log_debug( NULL, "[filter avcolor_space] %s -> %s @ %dx%d space %d\n",
 			mlt_image_format_name( *format ), mlt_image_format_name( output_format ),
-			width, height );
+			width, height, colorspace );
 
 		int in_fmt = convert_mlt_to_av_cs( *format );
 		int out_fmt = convert_mlt_to_av_cs( output_format );
@@ -154,7 +196,16 @@ static int convert_image( mlt_frame frame, uint8_t **image, mlt_image_format *fo
 		}
 
 		// Update the output
-		av_convert_image( output, *image, out_fmt, in_fmt, width, height );
+		if ( *format == mlt_image_yuv422 && mlt_properties_get( properties, "force_full_luma" )
+		     && ( output_format == mlt_image_rgb24 || output_format == mlt_image_rgb24a ) )
+		{
+			// By removing the frame property we only permit the luma to skip scaling once.
+			// Thereafter, we let swscale scale the luma range as it pleases since it seems
+			// we do not have control over the RGB to YUV conversion.
+			force_full_luma = mlt_properties_get_int( properties, "force_full_luma" );			
+			mlt_properties_set( properties, "force_full_luma", NULL );
+		}
+		av_convert_image( output, *image, out_fmt, in_fmt, width, height, colorspace, force_full_luma );
 		*image = output;
 		*format = output_format;
 		mlt_properties_set_data( properties, "image", output, size, mlt_pool_release, NULL );
@@ -193,12 +244,63 @@ static int convert_image( mlt_frame frame, uint8_t **image, mlt_image_format *fo
 	return error;
 }
 
+/* TODO: The below is not working because swscale does not have
+ * adjustable coefficients yet for RGB->YUV */
+
+static int get_image( mlt_frame frame, uint8_t **image, mlt_image_format *format, int *width, int *height, int writable )
+{
+	int error = 0;
+	mlt_profile profile = (mlt_profile) mlt_frame_pop_get_image( frame );
+	mlt_properties properties = MLT_FRAME_PROPERTIES(frame);
+	mlt_image_format format_from = *format;
+	mlt_image_format format_to = mlt_image_rgb24;
+	
+	error = mlt_frame_get_image( frame, image, format, width, height, writable );
+	
+	int frame_colorspace = mlt_properties_get_int( properties, "colorspace" );
+	
+	if ( !error && *format == mlt_image_yuv422 && profile->colorspace > 0 &&
+	     frame_colorspace > 0 && frame_colorspace != profile->colorspace )
+	{
+		mlt_log_debug( NULL, "[filter avcolor_space] colorspace %d -> %d\n",
+			frame_colorspace, profile->colorspace );
+		
+		// Convert to RGB using frame's colorspace
+		error = convert_image( frame, image, &format_from, format_to );
+
+		// Convert to YUV using profile's colorspace
+		if ( !error )
+		{
+			*image = mlt_properties_get_data( properties, "image", NULL );
+			format_from = mlt_image_rgb24;
+			format_to = *format;
+			mlt_properties_set_int( properties, "colorspace", profile->colorspace );
+			error = convert_image( frame, image, &format_from, format_to );
+			*image = mlt_properties_get_data( properties, "image", NULL );
+		}
+	}
+	
+	return error;
+}
+
 /** Filter processing.
 */
 
-static mlt_frame filter_process( mlt_filter this, mlt_frame frame )
+static mlt_frame filter_process( mlt_filter filter, mlt_frame frame )
 {
+	// Set a default colorspace on the frame if not yet set by the producer.
+	// The producer may still change it during get_image.
+	// This way we do not have to modify each producer to set a valid colorspace.
+	mlt_properties properties = MLT_FRAME_PROPERTIES(frame);
+	if ( mlt_properties_get_int( properties, "colorspace" ) <= 0 )
+		mlt_properties_set_int( properties, "colorspace", mlt_service_profile( MLT_FILTER_SERVICE(filter) )->colorspace );
+
 	frame->convert_image = convert_image;
+    
+//	Not working yet - see comment for get_image() above.
+//	mlt_frame_push_service( frame, mlt_service_profile( MLT_FILTER_SERVICE( filter ) ) );
+//	mlt_frame_push_get_image( frame, get_image );
+
 	return frame;
 }
 
