@@ -162,7 +162,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 			int count = ( *width - uneven ) / 2 + 1;
 			uint8_t y, u, v;
 
-			RGB2YUV( color.r, color.g, color.b, y, u, v );
+			RGB2YUV_601_SCALED( color.r, color.g, color.b, y, u, v );
 			i = *height + 1;
 			while ( --i )
 			{
@@ -221,6 +221,9 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	mlt_properties_set_data( properties, "image", *buffer, size, mlt_pool_release, NULL );
 	mlt_properties_set_data( properties, "alpha", alpha, alpha_size, mlt_pool_release, NULL );
 	mlt_properties_set_double( properties, "aspect_ratio", mlt_properties_get_double( producer_props, "aspect_ratio" ) );
+	mlt_properties_set_int( properties, "real_width", *width );
+	mlt_properties_set_int( properties, "real_height", *height );
+
 
 	return 0;
 }
